@@ -6,22 +6,26 @@ use IO::Handle;
 use Mojo::IOLoop;
 use Mojo::mysql::Results;
 use Mojo::mysql::Transaction;
+use Scalar::Util 'weaken';
 
 has [qw(dbh mysql)];
 has max_statements => 10;
 
 sub DESTROY {
   my $self = shift;
-  if ((my $dbh = $self->dbh) && (my $mysql = $self->mysql)) { $mysql->_enqueue($dbh) }
+  return unless my $dbh   = $self->dbh;
+  return unless my $mysql = $self->mysql;
+  $mysql->_enqueue($dbh, $self->{handle});
 }
 
 sub backlog { scalar @{shift->{waiting} || []} }
 
 sub begin {
   my $self = shift;
-  my $dbh  = $self->dbh;
-  $dbh->begin_work;
-  return Mojo::mysql::Transaction->new(dbh => $dbh);
+  $self->dbh->begin_work;
+  my $tx = Mojo::mysql::Transaction->new(db => $self);
+  weaken $tx->{db};
+  return $tx;
 }
 
 sub disconnect {
@@ -193,7 +197,7 @@ Disconnect database handle and prevent it from getting cached again.
 
 =head2 do
 
-  $db = $db->do('create table foo (bar varchar(255))');
+  $db = $db->do('create table foo (bar text)');
 
 Execute a statement and discard its result.
 
